/*	outback.h	version 0.92	Mar. 1, 2012
 *
 *	Written by Daniel Lloyd. 2011-2012.
 *	Property of OutBack Power Systems Inc.
 */


#ifndef _outback_h
#define _outback_h

/***********************************************************************/
/*
 * basic data types
 */
typedef	unsigned char			uint8;		/*  8 bits */
typedef unsigned short int		uint16;		/* 16 bits */
typedef unsigned int			uint32;		/* 32 bits */
typedef unsigned long long int	uint64;		/* 64 bits */

typedef char					int8;		/*  8 bits */
typedef short int				int16;		/* 16 bits */
typedef int						int32;		/* 32 bits */
typedef long long int			int64;		/* 64 bits */

typedef volatile int8			vint8;		/*  8 bits */
typedef volatile int16			vint16;		/* 16 bits */
typedef volatile int32			vint32;		/* 32 bits */
typedef volatile int64			vint64;		/* 64 bits */

typedef volatile uint8			vuint8;		/*  8 bits */
typedef volatile uint16			vuint16;	/* 16 bits */
typedef volatile uint32			vuint32;	/* 32 bits */
typedef volatile uint64			vuint64;	/* 64 bits */

/***********************************************************************/


#include "modbus.h"

#define MAX_DEVICES						(24)

#define SUNSPEC_COMMON_MODEL_BLOCK_DID		(0x0001)
#define SUNSPEC_AGGREGATOR_BLOCK_DID		(0x0002)
#define SUNSPEC_END_BLOCK_DID				(0xffff)
#define SUNSPEC_INVERTER_SINGLE_DID			(101)
#define SUNSPEC_INVERTER_SPLIT_DID			(102)
#define SUNSPEC_INVERTER_3PHASE_DID			(103)
#define SUNSPEC_BASIC_CC_DID				(64111)
#define SUNSPEC_OUTBACK_DID					(64110)
#define SUNSPEC_OUTBACK_FM_CC_DID			(64112)
#define SUNSPEC_OUTBACK_FX_DID				(64113)
#define SUNSPEC_OUTBACK_FX_CONFIG_DID		(64114)
#define SUNSPEC_OUTBACK_GS_SPLIT_DID		(64115)
#define SUNSPEC_OUTBACK_GS_CONFIG_DID		(64116)
#define SUNSPEC_OUTBACK_GS_SINGLE_DID		(64117)
#define SUNSPEC_OUTBACK_FNDC_DID			(64118)
#define SUNSPEC_OUTBACK_FNDC_CONFIG_DID		(64119)
#define SUNSPEC_OUTBACK_SYS_CONTROL_DID		(64120)


#define COMMON_MDOFFSET					(0)
#define	COMMON_FIELDS					(9)
#define	COMMON_SIZE						(69)
#define OUTBACK_MDOFFSET				(COMMON_MDOFFSET + COMMON_FIELDS)
#define	OUTBACK_FIELDS					(151)
#define	OUTBACK_SIZE					(407)
#define OB_SYS_CTRL_MDOFFSET			(OUTBACK_MDOFFSET + OUTBACK_FIELDS)
#define OB_SYS_CTRL_FIELDS				(25)
#define OB_SYS_CTRL_SIZE				(27)
#define CC_MDOFFSET						(OB_SYS_CTRL_MDOFFSET + OB_SYS_CTRL_FIELDS)
#define	CC_FIELDS						(28)
#define	CC_SIZE							(28)
#define CC_CONFIG_MDOFFSET				(CC_MDOFFSET + CC_FIELDS)

#define	CC_CONFIG_FIELDS				(74)
#define	CC_CONFIG_SIZE					(90)

#define I_MDOFFSET						(CC_CONFIG_MDOFFSET + CC_CONFIG_FIELDS)
#define I_FIELDS						(45)
#define I_SIZE							(52)
#define GS_CONFIG_MDOFFSET				(I_MDOFFSET + I_FIELDS)

#define GS_CONFIG_FIELDS				(68)
#define GS_CONFIG_SIZE					(84)

#define GS_SINGLE_MDOFFSET				(GS_CONFIG_MDOFFSET + GS_CONFIG_FIELDS)
#define GS_SINGLE_FIELDS				(41)
#define GS_SINGLE_SIZE					(41)
#define GS_SPLIT_MDOFFSET				(GS_SINGLE_MDOFFSET + GS_SINGLE_FIELDS)
#define GS_SPLIT_FIELDS					(53)
#define GS_SPLIT_SIZE					(53)
#define FX_CONFIG_MDOFFSET				(GS_SPLIT_MDOFFSET + GS_SPLIT_FIELDS)

#define FX_CONFIG_FIELDS				(58)
#define FX_CONFIG_SIZE					(74)

#define FX_MDOFFSET						(FX_CONFIG_MDOFFSET + FX_CONFIG_FIELDS)
#define FX_FIELDS						(31)
#define FX_SIZE							(31)
#define FNDC_CONFIG_MDOFFSET			(FX_MDOFFSET + FX_FIELDS)

#define FNDC_CONFIG_FIELDS				(38)
#define FNDC_CONFIG_SIZE				(54)

#define FNDC_MDOFFSET					(FNDC_CONFIG_MDOFFSET + FNDC_CONFIG_FIELDS)
#define FNDC_FIELDS						(76)
#define FNDC_SIZE						(78)
#define	AGGREGATOR_MDOFFSET				(FNDC_MDOFFSET + FNDC_FIELDS)
#define	AGGREGATOR_FIELDS				(12)
#define	AGGREGATOR_SIZE					(16)
#define END_MDOFFSET					(AGGREGATOR_MDOFFSET + AGGREGATOR_FIELDS)
#define	END_FIELDS						(2)
#define	END_SIZE						(2)
#define NULL_MDOFFSET					(0)
#define	NULL_FIELDS						(0)
#define	NULL_SIZE						(0)
#define	MAX_SIZE						(COMMON_SIZE + OUTBACK_SIZE + I_SIZE + MAX_HUB_DEVICES * (GS_SPLIT_SIZE + GS_CONFIG_SIZE) + END_SIZE)

#define C_STATUS_NORMAL					(0x00000000)	//Operating Normally
#define C_STATUS_ERROR					(0xFFFFFFFE)	//Generic Failure
#define C_STATUS_UNK					(0xFFFFFFFF)	//Status Unknown

// for OB_Control_Status

#define STATUS_ABSORB_ENABLED			0x0001
#define STATUS_FLOAT_ENABLED			0x0002
#define STATUS_AC_USE					0x0004
#define STATUS_INVERTER_OFF				0x0008
#define STATUS_INVERTER_SEARCH			0x0010
#define STATUS_INVERTER_ON				0x0020
#define STATUS_SELL_ENABLED				0x0040
#define STATUS_CHARGER_DISABLED			0x0080
#define STATUS_CHARGER_AUTO				0x0100
#define STATUS_CHARGER_BULK_AND_FLOAT	0x0200

#define OUTBACK_PASSWORD_SIZE			(8)
#define SMTP_SERVER_NAME_SIZE			(20)
#define SMTP_ACCOUNT_NAME_SIZE			(16)
#define SMTP_EMAIL_PASSWORD_SIZE		(8)
#define SMTP_EMAIL_USER_NAME_SIZE		(20)
#define STATUS_EMAIL_SUBJECT_SIZE		(25)
#define STATUS_EMAIL_TO_ADDR1_SIZE		(20)
#define STATUS_EMAIL_TO_ADDR2_SIZE		(20)
#define ALARM_EMAIL_SUBJECT_SIZE		(25)
#define ALARM_EMAIL_TO_ADDR1_SIZE		(20)
#define ALARM_EMAIL_TO_ADDR2_SIZE		(20)
#define FTP_PASSWORD_SIZE				(8)
#define TELNET_PASSWORD_SIZE			(8)
#define TIME_SERVER_NAME_SIZE			(20)

#define ERR_LAST_WRITE_LIMIT_HIGH			(0)
#define ERR_LAST_WRITE_LIMIT_LOW			(1)
#define ERR_LAST_WRITE_INVALID				(2)
#define ERR_DHCP_FAILED						(3)
#define ERR_DNS_RESOLVE_FAILED				(4)
#define ERR_SMTP_AUTH_FAILED				(5)
#define ERR_SMTP_SEND_FAILED				(6)
#define ERR_FTP_ERROR						(7)
#define ERR_SD_CARD_ERROR					(8)
#define ERR_SNTP_FAILED						(9)
#define ERR_WRITE_WHILE_LOCKED				(10)
#define	ERR_DEVICE_FW_UPDATE_INVALID		(11) 
#define ERR_DEVICE_FW_UPDATE_FILE_NOT_FOUND	(12) 
#define ERR_DEVICE_FW_UPDATE_FILE_INVALID	(13) 
#define ERR_DEVICE_FW_UPDATE_FAILURE		(14) 
#define ERR_OPERATION_NOT_SUPPORTED			(15) 


#define OUTBACK_READ_ERROR(x,y)			(((x) & (0x0001 << (y))) >> (y))
#define OUTBACK_DEVICE_ERROR(x)			(((x) & 0xF000) >> 12)


typedef enum {
	C_SunSpec_ID,											//	0
	C_SunSpec_DID,											//	1
	C_SunSpec_Length,										//	2
	C_Manufacturer,											//	3
	C_Model,												//	4
	C_Options,												//	5
	C_Version,												//	6
	C_SerialNumber,											//	7
	C_DeviceAddress,										//	8
	
	OutBack_SunSpec_DID,									//	9
	OutBack_SunSpec_Length,									//	10
	OutBack_Major_Firmware_Number,							//	11
	OutBack_Mid_Firmware_Number,							//	12
	OutBack_Minor_Firmware_Number,							//	13
	OutBack_Encryption_Key,									//	14
	OutBack_MAC_Address,									//  15
	OutBack_Write_Password,									//	16
	OutBack_Enable_DHCP,									//	17
	OutBack_TCPIP_Address,									//	18
	OutBack_TCPIP_Gateway,									//	19
	OutBack_TCPIP_Netmask,									//	20
	OutBack_TCPIP_DNS_1,									//	21
	OutBack_TCPIP_DNS_2,									//	22
	OutBack_Modbus_Port,									//	23
	OutBack_SMTP_Server_Name,								//	24
	OutBack_SMTP_Account_Name,								//	25
	OutBack_SMTP_SSL_Enable,								//	26
	OutBack_SMTP_Email_Password,							//	27
	OutBack_SMTP_Email_User_Name,							//	28
	OutBack_Status_Email_Interval,							//	29
	OutBack_Status_Email_Status_Time,						//	30
	OutBack_Status_Email_Subject_Line,						//	31
	OutBack_Status_Email_To_Address_1,						//	32
	OutBack_Status_Email_To_Address_2,						//	33
	OutBack_Alarm_Email_Enable,								//	34
	OutBack_Alarm_Email_Subject_Line,						//	35
	OutBack_Alarm_Email_To_Address_1,						//	36
	OutBack_Alarm_Email_To_Address_2,						//	37
	OutBack_FTP_Password,									//	38
	OutBack_Telnet_Password,								//	39
	OutBack_SD_Card_Log_Write_Interval,						//	40
	OutBack_SD_Card_Log_Retain_Days,						//	41
	OutBack_SD_Card_Logging_Mode,							//	42
	OutBack_Time_Server_Name,								//	43
	OutBack_Enable_Time_Server,								//	44
	OutBack_Set_Time_Zone,									//	45
	OutBack_Enable_Float_Coordination,						//	46
	OutBack_Enable_FNDC_Charge_Termination,					//	47
	OutBack_Enable_FNDC_Grid_Tie_Control,					//	48
	OutBack_Voltage_SF,										//  49
	OutBack_Hour_SF,										//  50
	OutBack_AGS_Mode,										//  51
	OutBack_AGS_Port,										//  52
	OutBack_AGS_Port_Type,									//  53
	OutBack_Generator_Type,									//  54
	OutBack_AGS_DC_Gen_Absorb_Voltage,						//  55  						
	OutBack_AGS_DC_Gen_Absorb_Time,                         //  56  
	OutBack_AGS_Fault_Time,                                 //  57  
	OutBack_AGS_Gen_Cool_Down_Time,                         //  58  
	OutBack_AGS_Gen_Warm_Up_Time,                           //  59  
	OutBack_Generator_Exercise_Mode,                        //  60  
	OutBack_Exercise_Start_Hour,                            //  61  
	OutBack_Exercise_Start_Minute,                          //  62  
	OutBack_Exercise_Day,                                   //  63  
	OutBack_Exercise_Period,                                //  64  
	OutBack_Exercise_Interval,                              //  65  
	OutBack_AGS_Sell_Mode,                                  //  66  
	OutBack_AGS_2_Min_Start_Mode,                           //  67  
	OutBack_AGS_2_Min_Start_Voltage,                        //  68  
	OutBack_AGS_2_Hour_Start_Mode,                          //  69  
	OutBack_AGS_2_Hour_Start_Voltage,                       //  70  
	OutBack_AGS_24_Hour_Start_Mode,                         //  71  
	OutBack_AGS_24_Hour_Start_Voltage,                      //  72  
	OutBack_AGS_Load_Start_Mode,                            //  73  
	OutBack_AGS_Load_Start_kW,                              //  74  
	OutBack_AGS_Load_Start_Delay,                           //  75  
	OutBack_AGS_Load_Stop_kW,                               //  76  
	OutBack_AGS_Load_Stop_Delay,                            //  77  
	OutBack_AGS_SOC_Start_Mode,                             //  78  
	OutBack_AGS_SOC_Start_Percentage,                       //  79  
	OutBack_AGS_SOC_Stop_Percentage,                        //  80  
	OutBack_AGS_Enable_Full_Charge_Mode,                    //  81  
	OutBack_AGS_Full_Charge_Interval,                       //  82  
	OutBack_AGS_Must_Run_Mode,                              //  83  
	OutBack_AGS_Must_Run_Weekday_Start_Hour,                //  84  
	OutBack_AGS_Must_Run_Weekday_Start_Minute,              //  85  
	OutBack_AGS_Must_Run_Weekday_Stop_Hour,                 //  86  
	OutBack_AGS_Must_Run_Weekday_Stop_Minute,               //  87  
	OutBack_AGS_Must_Run_Weekend_Start_Hour,                //  88  
	OutBack_AGS_Must_Run_Weekend_Start_Minute,              //  89  
	OutBack_AGS_Must_Run_Weekend_Stop_Hour,                 //  90  
	OutBack_AGS_Must_Run_Weekend_Stop_Minute,               //  91  
	OutBack_AGS_Quiet_Time_Mode,                            //  92  
	OutBack_AGS_Quiet_Time_Weekday_Start_Hour,              //  93  
	OutBack_AGS_Quiet_Time_Weekday_Start_Minute,            //  94  
	OutBack_AGS_Quiet_Time_Weekday_Stop_Hour,               //  95  
	OutBack_AGS_Quiet_Time_Weekday_Stop_Minute,             //  96  
	OutBack_AGS_Quiet_Time_Weekend_Start_Hour,              //  97  
	OutBack_AGS_Quiet_Time_Weekend_Start_Minute,            //  98  
	OutBack_AGS_Quiet_Time_Weekend_Stop_Hour,               //  99  
	OutBack_AGS_Quiet_Time_Weekend_Stop_Minute,             // 100  
	OutBack_AGS_Total_Generator_Run_Time,                   // 101  
	OutBack_HBX_Mode,                                       // 102  
	OutBack_HBX_Grid_Connect_Voltage,                       // 103  
	OutBack_HBX_Grid_Connect_Delay,                         // 104  
	OutBack_HBX_Grid_Disconnect_Voltage,                    // 105  
	OutBack_HBX_Grid_Disconnect_Delay,                      // 106  
	OutBack_HBX_Grid_Connect_SOC,                           // 107  
	OutBack_HBX_Grid_Disconnect_SOC,                        // 108  
	OutBack_Grid_Use_Interval_1_Mode,                       // 109  
	OutBack_Grid_Use_Interval_1_Weekday_Start_Hour,         // 110  
	OutBack_Grid_Use_Interval_1_Weekday_Start_Minute,       // 111  
	OutBack_Grid_Use_Interval_1_Weekday_Stop_Hour,          // 112  
	OutBack_Grid_Use_Interval_1_Weekday_Stop_Minute,        // 113  
	OutBack_Grid_Use_Interval_1_Weekend_Start_Hour,         // 114  
	OutBack_Grid_Use_Interval_1_Weekend_Start_Minute,       // 115  
	OutBack_Grid_Use_Interval_1_Weekend_Stop_Hour,          // 116  
	OutBack_Grid_Use_Interval_1_Weekend_Stop_Minute,        // 117  
	OutBack_Grid_Use_Interval_2_Mode,                       // 118  
	OutBack_Grid_Use_Interval_2_Weekday_Start_Hour,         // 119  
	OutBack_Grid_Use_Interval_2_Weekday_Start_Minute,       // 120  
	OutBack_Grid_Use_Interval_2_Weekday_Stop_Hour,          // 121  
	OutBack_Grid_Use_Interval_2_Weekday_Stop_Minute,        // 122  
	OutBack_Grid_Use_Interval_3_Mode,                       // 123  
	OutBack_Grid_Use_Interval_3_Weekday_Start_Hour,         // 124  
	OutBack_Grid_Use_Interval_3_Weekday_Start_Minute,       // 125  
	OutBack_Grid_Use_Interval_3_Weekday_Stop_Hour,          // 126  
	OutBack_Grid_Use_Interval_3_Weekday_Stop_Minute,        // 127  
	OutBack_Load_Grid_Transfer_Mode,                        // 128  
	OutBack_Load_Grid_Transfer_Threshold,                   // 129  
	OutBack_Load_Grid_Transfer_Connect_Delay,               // 130  
	OutBack_Load_Grid_Transfer_Disconnect_Delay,            // 131  
	OutBack_Load_Grid_Transfer_Connect_Battery_Voltage,     // 132  
	OutBack_Load_Grid_Transfer_Re_Connect_Battery_Voltage,  // 133
	OutBack_Global_Charger_Control_Mode,                    // 134  
	OutBack_Global_Charger_Output_Limit,                    // 135  
	OutBack_Radian_AC_Coupled_Mode,                         // 136  
	OutBack_Radian_AC_Coupled_AUX_Port,                     // 137  
	OutBack_URL_Lock,                                       // 138  
	OutBack_Web_Reporting_Base_URL,                         // 139  
	OutBack_Web_User_Logged_In_Status,						// 140  
	OutBack_HUB_Type,										// 141  
	OutBack_HUB_Major_Firmware_Number,                      // 142  
	OutBack_HUB_Mid_Firmware_Number,                        // 143  
	OutBack_HUB_Minor_Firmware_Number,	                    // 144  
	OutBack_Year,										    // 145
	OutBack_Month,										    // 146
	OutBack_Day,										    // 147
	OutBack_Hour,										    // 148
	OutBack_Minute,										    // 149
	OutBack_Second,										    // 150
	OutBack_Temp_Batt,									    // 151
	OutBack_Temp_Ambient,								    // 152
	OutBack_Temp_SF,									    // 153
	OutBack_Error,										    // 154
	OutBack_Status,										    // 155
	OutBack_Update_Device_Firmware_Port,				    // 156
	OutBack_Gateway_Type,									// 157
	OutBack_System_Voltage,									// 158
	OutBack_Measured_System_Voltage,						// 159
	
	OB_SunSpec_DID,										    // 160
	OB_SunSpec_Length,									    // 161
	OB_DC_Voltage_SF,									    // 162
	OB_AC_Current_SF,									    // 163
	OB_Time_SF,											    // 164
	OB_Bulk_Charge_Enable_Disable,						    // 165
	OB_Inverter_AC_Drop_Use,							    // 166
	OB_Set_Inverter_Mode,								    // 167
	OB_Grid_Tie_Mode,									    // 168
	OB_Set_Inverter_Charger_Mode,						    // 169
	OB_Control_Status,									    // 170
	OB_Set_Sell_Voltage,								    // 171
	OB_Set_Radian_Inverter_Sell_Current_Limit,			    // 172
	OB_Set_Absorb_Voltage,								    // 173
	OB_Set_Absorb_Time,									    // 174
	OB_Set_Float_Voltage,								    // 175
	OB_Set_Float_Time,									    // 176
	OB_Set_Inverter_Charger_Current_Limit,				    // 177
	OB_Set_Inverter_AC1_Current_Limit,					    // 178
	OB_Set_Inverter_AC2_Current_Limit,					    // 179
	OB_AGS_Operational_Mode,								// 180       
	OB_AGS_Operational_State,								// 181
	OB_AGS_Operational_State_Timer,							// 182
	OB_Gen_Last_Run_Start_Time_GMT,							// 183
	OB_Gen_Last_Run_Duration,								// 184
	
	CC_SunSpec_DID,											// 185  
	CC_SunSpec_Length,										// 186 
	CC_port_number,											// 187  
	CC_Voltage_SF,											// 188  
	CC_Current_SF,											// 189  
	CC_Power_SF,											// 190  
	CC_AH_SF,												// 191  
	CC_KWH_SF,												// 192  
	CC_Batt_Voltage,										// 193  
	CC_Array_Voltage,										// 194  
	CC_Batt_Current,										// 195  
	CC_Array_Current,										// 196  
	CC_Charger_State,										// 197  
	CC_Watts,												// 198  
	CC_Todays_Min_Battery_Volts,							// 199  
	CC_Todays_Max_Battery_Volts,							// 200  
	CC_VOC,													// 201  
	CC_Todays_Peak_VOC,										// 202  
	CC_Todays_kWH,											// 203  
	CC_Todays_AH,											// 204  
	CC_Lifetime_kWH_Hours,									// 205  
	CC_Lifetime_kAmp_Hours,									// 206  
	CC_Lifetime_Max_Watts,									// 207  
	CC_Lifetime_Max_Battery_Volts,							// 208  
	CC_Lifetime_Max_VOC,									// 209  
	CC_Temp_SF,												// 210  
	CC_Temp_Output_FETs,									// 211  
	CC_Temp_Enclosure,										// 212
	  
	CCconfig_SunSpec_DID,								    // 213
	CCconfig_SunSpec_Length,							    // 214
	CCconfig_port_number,								    // 215
	CCconfig_Voltage_SF,								    // 216
	CCconfig_Current_SF,								    // 217
	CCconfig_Hours_SF,									    // 218
	CCconfig_Power_SF,									    // 219
	CCconfig_AH_SF,										    // 220
	CCconfig_KWH_SF,									    // 221
	CCconfig_Faults,									    // 222
	CCconfig_Absorb_Volts,								    // 223
	CCconfig_Absorb_Time_Hours,							    // 224
	CCconfig_Absorb_End_Amps,							    // 225
	CCconfig_Rebulk_Volts,								    // 226
	CCconfig_Float_Volts,								    // 227
	CCconfig_Bulk_Current,								    // 228
	CCconfig_EQ_Volts,									    // 229
	CCconfig_EQ_Time_Hours,								    // 230
	CCconfig_Auto_EQ_Days,								    // 231
	CCconfig_MPPT_Mode,									    // 232
	CCconfig_Sweep_Width,								    // 233
	CCconfig_Sweep_Max_Percentage,						    // 234
	CCconfig_U_Pick_PWM_Duty_Cycle,						    // 235
	CCconfig_Grid_Tie_Mode,								    // 236
	CCconfig_Temp_Comp_Mode,							    // 237
	CCconfig_Temp_Comp_Lower_Limit_Volts,				    // 238
	CCconfig_Temp_Comp_Upper_Limit_Volts,				    // 239
	CCconfig_Temp_Comp_Slope,							    // 240
	CCconfig_Auto_Restart_Mode,							    // 241
	CCconfig_Wakeup_VOC,								    // 242
	CCconfig_Snooze_Mode_Amps,							    // 243
	CCconfig_Wakeup_Interval,							    // 244
	CCconfig_AUX_Mode,									    // 245
	CCconfig_AUX_Control,								    // 246
	CCconfig_AUX_State,									    // 247
	CCconfig_AUX_Polarity,								    // 248
	CCconfig_AUX_Low_Batt_Disconnect,					    // 249
	CCconfig_AUX_Low_Batt_Reconnect,					    // 250
	CCconfig_AUX_Low_Batt_Disconnect_Delay,				    // 251
	CCconfig_AUX_Vent_Fan_Volts,						    // 252
	CCconfig_AUX_PV_Limit_Volts,						    // 253
	CCconfig_AUX_PV_Limit_Hold_Time,					    // 254
	CCconfig_AUX_Night_Light_Thres_Volts,				    // 255 
	CCconfig_Night_Light_ON_Hours,						    // 256
	CCconfig_Night_Light_ON_Hyst_Time,					    // 257
	CCconfig_Night_Light_OFF_Hyst_Time,					    // 258
	CCconfig_AUX_Error_Battery_Volts,					    // 259
	CCconfig_AUX_Divert_Hold_Time,						    // 260
	CCconfig_AUX_Divert_Delay_Time,						    // 261
	CCconfig_AUX_Divert_Relative_Volts,					    // 262
	CCconfig_AUX_Divert_Hyst_Volts,						    // 263
	CCconfig_Major_Firmware_Number,						    // 264
	CCconfig_Mid_Firmware_Number,						    // 265
	CCconfig_Minor_Firmware_Number,						    // 266
	CCconfig_Set_Log_Day_Offset,						    // 267
	CCconfig_Get_Current_Log_Day_Offset,				    // 268
	CCconfig_Log_Daily_AH,								    // 269
	CCconfig_Log_Daily_kWH,								    // 270
	CCconfig_Log_Daily_Max_Output_Amps,					    // 271
	CCconfig_Log_Daily_Max_Output_Watts,				    // 272
	CCconfig_Log_Daily_Absorb_Time,						    // 273
	CCconfig_Log_Daily_Float_Time,						    // 274
	CCconfig_Log_Daily_Min_Batt_Volts,					    // 275
	CCconfig_Log_Daily_Max_Batt_Volts,					    // 276
	CCconfig_Log_Daily_Max_Input_Volts,					    // 277
	CCconfig_Clear_Log_Read,							    // 278
	CCconfig_Clear_Log_Write_Complement,				    // 279
	CCconfig_Stats_Maximum_Reset_Read,					    // 280
	CCconfig_Stats_Maximum_Write_Complement,			    // 281
	CCconfig_Stats_Totals_Reset_Read,					    // 282
	CCconfig_Stats_Totals_Write_Complement,				    // 283
	CCconfig_Battery_Voltage_Calibrate_Offset,				// 284
	CCconfig_Serial_Number,								    // 285
	CCconfig_Model_Number,								    // 286
	
	I_SunSpec_DID,										    // 287
	I_SunSpec_Length,									    // 288
	I_AC_Current,										    // 289
	I_AC_CurrentA,										    // 290
	I_AC_CurrentB,										    // 291
	I_AC_CurrentC,										    // 292
	I_AC_Current_SF,									    // 293
	I_AC_VoltageAB,										    // 294
	I_AC_VoltageBC,										    // 295
	I_AC_VoltageCA,										    // 296
	I_AC_VoltageAN,										    // 297
	I_AC_VoltageBN,										    // 298
	I_AC_VoltageCN,										    // 299
	I_AC_Voltage_SF,									    // 300
	I_AC_Power,											    // 301
	I_AC_Power_SF,										    // 302
	I_AC_Frequency,										    // 303
	I_AC_Frequency_SF,									    // 304
	I_AC_VA,											    // 305
	I_AC_VA_SF,											    // 306
	I_AC_VAR,											    // 307
	I_AC_VAR_SF,										    // 308
	I_AC_PF,											    // 309
	I_AC_PF_SF,											    // 310
	I_AC_Energy_WH,										    // 311
	I_AC_Energy_WH_SF,									    // 312
	I_DC_Current,										    // 313
	I_DC_Current_SF,									    // 314
	I_DC_Voltage,										    // 315
	I_DC_Voltage_SF,									    // 316
	I_DC_Power,											    // 317
	I_DC_Power_SF,										    // 318
	I_Temp_Cab,											    // 319
	I_Temp_Sink,										    // 320
	I_Temp_Trans,										    // 321
	I_Temp_Other,										    // 322
	I_Temp_SF,											    // 323
	I_Status,											    // 324
	I_Status_Vendor,									    // 325
	I_Event_1,											    // 326
	I_Event_2,											    // 327
	I_Event_1_Vendor,									    // 328
	I_Event_2_Vendor,									    // 329
	I_Event_3_Vendor,									    // 330
	I_Event_4_Vendor,									    // 331
	
	GSconfig_SunSpec_DID,								    // 332
	GSconfig_SunSpec_Length,							    // 333
	GSconfig_Port_Number,								    // 334
	GSconfig_DC_Voltage_SF,								    // 335
	GSconfig_AC_Current_SF,								    // 336
	GSconfig_AC_Voltage_SF,								    // 337
	GSconfig_Time_SF,									    // 338
	GSconfig_Major_Firmware_Number,						    // 339
	GSconfig_Mid_Firmware_Number,						    // 340
	GSconfig_Minor_Firmware_Number,						    // 341
	GSconfig_Absorb_Volts,								    // 342
	GSconfig_Absorb_Time_Hours,							    // 343
	GSconfig_Float_Volts,								    // 344
	GSconfig_Float_Time_Hours,							    // 345
	GSconfig_ReFloat_Volts,								    // 346
	GSconfig_EQ_Volts,									    // 347
	GSconfig_EQ_Time_Hours,								    // 348
	GSconfig_Search_Sensitivity,						    // 349
	GSconfig_Search_Pulse_Length,						    // 350
	GSconfig_Search_Pulse_Spacing,						    // 351
	GSconfig_AC_Input_Select_Priority,					    // 352
	GSconfig_Grid_AC_Input_Current_Limit,				    // 353
	GSconfig_Gen_AC_Input_Current_Limit,				    // 354
	GSconfig_Charger_AC_Input_Current_Limit,			    // 355
	GSconfig_Charger_Operating_Mode,					    // 356
	GSconfig_AC_Coupled,								    // 357
	GSconfig_Grid_Input_Mode,							    // 358
	GSconfig_Grid_Lower_Input_Voltage_Limit,			    // 359
	GSconfig_Grid_Upper_Input_Voltage_Limit,			    // 360
	GSconfig_Grid_Transfer_Delay,						    // 361
	GSconfig_Grid_Connect_Delay,						    // 362
	GSconfig_Gen_Input_Mode,							    // 363
	GSconfig_Gen_Lower_Input_Voltage_Limit,				    // 364
	GSconfig_Gen_Upper_Input_Voltage_Limit,				    // 365
	GSconfig_Gen_Transfer_Delay,						    // 366
	GSconfig_Gen_Connect_Delay,							    // 367
	GSconfig_AC_Output_Voltage,							    // 368
	GSconfig_Low_Battery_Cut_Out_Voltage,				    // 369
	GSconfig_Low_Battery_Cut_In_Voltage,				    // 370
	GSconfig_AUX_Mode,									    // 371
	GSconfig_AUX_Control,								    // 372
	GSconfig_AUX_ON_Battery_Voltage,					    // 373
	GSconfig_AUX_ON_Delay_Time,							    // 374
	GSconfig_AUX_OFF_Battery_Voltage,					    // 375
	GSconfig_AUX_OFF_Delay_Time,						    // 376
	GSconfig_AUX_Relay_Mode,							    // 377
	GSconfig_AUX_Relay_Control,							    // 378
	GSconfig_AUX_Relay_ON_Battery_Voltage,				    // 379
	GSconfig_AUX_Relay_ON_Delay_Time,					    // 380
	GSconfig_AUX_Relay_OFF_Battery_Voltage,				    // 381
	GSconfig_AUX_Relay_OFF_Delay_Time,					    // 382
	GSconfig_Stacking_Mode,								    // 383
	GSconfig_Master_Power_Save_Level,					    // 384
	GSconfig_Slave_Power_Save_Level,					    // 385
	GSconfig_Sell_Volts,								    // 386
	GSconfig_Grid_Tie_Window,							    // 387
	GSconfig_Grid_Tie_Enable,							    // 388
	GSconfig_Grid_AC_Input_Voltage_Cal_Factor,				// 389
	GSconfig_Gen_AC_Input_Voltage_Cal_Factor,				// 390
	GSconfig_AC_Output_Voltage_Cal_Factor,				    // 391
	GSconfig_Battery_Voltage_Cal_Factor,				    // 392	
	GSconfig_ReBulk_Volts,								    // 393
	GSconfig_Mini_Grid_LBX_Volts,						    // 394
	GSconfig_Mini_Grid_LBX_Delay,						    // 395
	GSconfig_Grid_Zero_DoD_Volts,						    // 396
	GSconfig_Grid_Zero_DoD_Max_Offset_AC_Amps,			    // 397			  
	GSconfig_Serial_Number,								    // 398
	GSconfig_Model_Number,								    // 399
	
	GS_Single_SunSpec_DID,								    // 400
	GS_Single_SunSpec_Length,							    // 401
	GS_Single_Port_Number,								    // 402
	GS_Single_DC_Voltage_SF,							    // 403
	GS_Single_AC_Current_SF,							    // 404
	GS_Single_AC_Voltage_SF,							    // 405
	GS_Single_Frequency_SF,								    // 406
	GS_Single_Inverter_Output_Current,					    // 407
	GS_Single_Inverter_Charge_Current,					    // 408
	GS_Single_Inverter_Buy_Current,						    // 409
	GS_Single_Inverter_Sell_Current,					    // 410
	GS_Single_Grid_Input_AC_Voltage,					    // 411
	GS_Single_Gen_Input_AC_Voltage,						    // 412
	GS_Single_AC_Output_Voltage,						    // 413
	GS_Single_Inverter_Operating_Mode,					    // 414
	GS_Single_Error_Flags,								    // 415
	GS_Single_Warning_Flags,							    // 416
	GS_Single_Battery_Voltage,							    // 417
	GS_Single_Temp_Comp_Target_Voltage,					    // 418
	GS_Single_AUX_Output_State,							    // 419
	GS_Single_AUX_Relay_Output_State,					    // 420
	GS_Single_L_Module_Transformer_Temp,				    // 421
	GS_Single_L_Module_Capacitor_Temp,					    // 422
	GS_Single_L_Module_FET_Temp,						    // 423
	GS_Single_R_Module_Transformer_Temp,				    // 424
	GS_Single_R_Module_Capacitor_Temp,					    // 425
	GS_Single_R_Module_FET_Temp,						    // 426
	GS_Single_Battery_Temperature,						    // 427
	GS_Single_AC_Input_Selection,						    // 428
	GS_Single_AC_Input_Frequency,						    // 429
	GS_Single_AC_Input_Voltage,							    // 430
	GS_Single_AC_Input_State,							    // 431
	GS_Single_Minimum_AC_Input_Voltage,					    // 432
	GS_Single_Maximum_AC_Input_Voltage,					    // 433
	GS_Single_Sell_Status,								    // 434
	GS_Single_kWh_SF,									    // 435	
	GS_Single_AC1_Buy_kWh,									// 436
	GS_Single_AC2_Buy_kWh,									// 437
	GS_Single_AC1_Sell_kWh,									// 438
	GS_Single_AC2_Sell_kWh,									// 439
	GS_Single_Output_kWh,								    // 440
	
	GS_Split_SunSpec_DID,								    // 441
	GS_Split_SunSpec_Length,							    // 442
	GS_Split_Port_Number,								    // 443
	GS_Split_DC_Voltage_SF,								    // 444
	GS_Split_AC_Current_SF,								    // 445
	GS_Split_AC_Voltage_SF,								    // 446
	GS_Split_Frequency_SF,								    // 447
	GS_Split_L1_Inverter_Output_Current,				    // 448
	GS_Split_L1_Inverter_Charge_Current,				    // 449
	GS_Split_L1_Inverter_Buy_Current,					    // 450
	GS_Split_L1_Inverter_Sell_Current,					    // 451
	GS_Split_L1_Grid_Input_AC_Voltage,					    // 452
	GS_Split_L1_Gen_Input_AC_Voltage,					    // 453
	GS_Split_L1_AC_Output_Voltage,						    // 454
	GS_Split_L2_Inverter_Output_Current,				    // 455
	GS_Split_L2_Inverter_Charge_Current,				    // 456
	GS_Split_L2_Inverter_Buy_Current,					    // 457
	GS_Split_L2_Inverter_Sell_Current,					    // 458
	GS_Split_L2_Grid_Input_AC_Voltage,					    // 459
	GS_Split_L2_Gen_Input_AC_Voltage,					    // 460
	GS_Split_L2_AC_Output_Voltage,						    // 461
	GS_Split_Inverter_Operating_Mode,					    // 462
	GS_Split_Error_Flags,								    // 463
	GS_Split_Warning_Flags,								    // 464
	GS_Split_Battery_Voltage,							    // 465
	GS_Split_Temp_Comp_Target_Voltage,					    // 466
	GS_Split_AUX_Output_State,							    // 467
	GS_Split_AUX_Relay_Output_State,					    // 468
	GS_Split_L_Module_Transformer_Temp,					    // 469
	GS_Split_L_Module_Capacitor_Temp,					    // 470
	GS_Split_L_Module_FET_Temp,							    // 471
	GS_Split_R_Module_Transformer_Temp,					    // 472
	GS_Split_R_Module_Capacitor_Temp,					    // 473
	GS_Split_R_Module_FET_Temp,							    // 474
	GS_Split_Battery_Temperature,						    // 475
	GS_Split_AC_Input_Selection,						    // 476
	GS_Split_AC_Input_Frequency,						    // 477
	GS_Split_AC_Input_Voltage,							    // 478
	GS_Split_AC_Input_State,							    // 479
	GS_Split_Minimum_AC_Input_Voltage,					    // 480
	GS_Split_Maximum_AC_Input_Voltage,					    // 481
	GS_Split_Sell_Status,								    // 482
	GS_Split_kWh_SF,									    // 483
	GS_Split_AC1_L1_Buy_kWh,								// 484
	GS_Split_AC2_L1_Buy_kWh,								// 485
	GS_Split_AC1_L1_Sell_kWh,								// 486
	GS_Split_AC2_L1_Sell_kWh,								// 487
	GS_Split_L1_Output_kWh,									// 488
	GS_Split_AC1_L2_Buy_kWh,								// 489
	GS_Split_AC2_L2_Buy_kWh,								// 490
	GS_Split_AC1_L2_Sell_kWh,								// 491
	GS_Split_AC2_L2_Sell_kWh,								// 492
	GS_Split_L2_Output_kWh,									// 493
	
	FXconfig_SunSpec_DID,								    // 494
	FXconfig_SunSpec_Length,							    // 495
	FXconfig_Port_Number,								    // 496
	FXconfig_DC_Voltage_SF,								    // 497
	FXconfig_AC_Current_SF,								    // 498
	FXconfig_AC_Voltage_SF,								    // 499
	FXconfig_Time_SF,									    // 500 
	FXconfig_Major_Firmware_Number,						    // 501
	FXconfig_Mid_Firmware_Number,						    // 502
	FXconfig_Minor_Firmware_Number,						    // 503
	FXconfig_Absorb_Volts,								    // 504
	FXconfig_Absorb_Time_Hours,							    // 505
	FXconfig_Float_Volts,								    // 506
	FXconfig_Float_Time_Hours,							    // 507
	FXconfig_ReFloat_Volts,								    // 508
	FXconfig_EQ_Volts,									    // 509
	FXconfig_EQ_Time_Hours,								    // 510
	FXconfig_Search_Sensitivity,						    // 511
	FXconfig_Search_Pulse_Length,						    // 512 
	FXconfig_Search_Pulse_Spacing,						    // 513
	FXconfig_AC_Input_Type,								    // 514
	FXconfig_Input_Support,								    // 515
	FXconfig_Grid_AC_Input_Current_Limit,				    // 516
	FXconfig_Gen_AC_Input_Current_Limit,				    // 517
	FXconfig_Charger_AC_Input_Current_Limit,			    // 518
	FXconfig_Charger_Operating_Mode,					    // 519
	FXconfig_Grid_Lower_Input_Voltage_Limit,			    // 520
	FXconfig_Grid_Upper_Input_Voltage_Limit,			    // 521
	FXconfig_Grid_Transfer_Delay,						    // 522
	FXconfig_Gen_Lower_Input_Voltage_Limit,				    // 523
	FXconfig_Gen_Upper_Input_Voltage_Limit,				    // 524
	FXconfig_Gen_Transfer_Delay,						    // 525
	FXconfig_Gen_Connect_Delay,							    // 526
	FXconfig_AC_Output_Voltage,							    // 527
	FXconfig_Low_Battery_Cut_Out_Voltage,				    // 528 
	FXconfig_Low_Battery_Cut_In_Voltage,				    // 529
	FXconfig_AUX_Mode,									    // 530
	FXconfig_AUX_Control,								    // 531
	FXconfig_AUX_Load_Shed_Enable_Voltage,				    // 532
	FXconfig_AUX_Gen_Alert_On_Voltage,					    // 533
	FXconfig_AUX_Gen_Alert_On_Delay,					    // 534
	FXconfig_AUX_Gen_Alert_Off_Voltage,					    // 535
	FXconfig_AUX_Gen_Alert_Off_Delay,					    // 536
	FXconfig_AUX_Vent_Fan_Enable_Voltage,				    // 537
	FXconfig_AUX_Vent_Fan_Off_Period,					    // 538
	FXconfig_AUX_Divert_Enable_Voltage,					    // 539
	FXconfig_AUX_Divert_Off_Delay,						    // 540
	FXconfig_Stacking_Mode,								    // 541
	FXconfig_Master_Power_Save_Level,					    // 542
	FXconfig_Slave_Power_Save_Level,					    // 543
	FXconfig_Sell_Volts,								    // 544
	FXconfig_Grid_Tie_Window,							    // 545
	FXconfig_Grid_Tie_Enable,							    // 546
	FXconfig_AC_Input_Voltage_Calibrate_Factor,			    // 547
	FXconfig_AC_Output_Voltage_Calibrate_Factor,		    // 548
	FXconfig_Battery_Voltage_Calibrate_Factor,			    // 549
	FXconfig_Serial_Number,								    // 550
	FXconfig_Model_Number,								    // 551
	
	FX_SunSpec_DID,										    // 552
	FX_SunSpec_Length,									    // 553
	FX_Port_Number,										    // 554
	FX_DC_Voltage_SF,									    // 555
	FX_AC_Current_SF,									    // 556
	FX_AC_Voltage_SF,									    // 557
	FX_AC_Frequency_SF,									    // 558
	FX_Inverter_Output_Current,							    // 559
	FX_Inverter_Charge_Current,							    // 560
	FX_Inverter_Buy_Current,							    // 561
	FX_Inverter_Sell_Current,							    // 562
	FX_Output_AC_Voltage,								    // 563
	FX_Inverter_Operating_Mode,							    // 564 
	FX_Error_Flags,										    // 565
	FX_Warning_Flags,									    // 566
	FX_Battery_Voltage,									    // 567
	FX_Temp_Compensated_Target_Voltage,					    // 568
	FX_AUX_Output_State,								    // 569
	FX_Transformer_Temperature,							    // 570
	FX_Capacitor_Temperature,							    // 571
	FX_FET_Temperature,									    // 572
	FX_AC_Input_Frequency,								    // 573
	FX_AC_Input_Voltage,								    // 574
	FX_AC_Input_State,									    // 575
	FX_Minimum_AC_Input_Voltage,						    // 576
	FX_Maximum_AC_Input_Voltage,						    // 577
	FX_Sell_Status,										    // 578
	FX_kWh_SF,											    // 579
	FX_Buy_kWh,											    // 580
	FX_Sell_kWh,										    // 581
	FX_Output_kWh,										    // 582
	
	FNconfig_SunSpec_DID,								    // 583
	FNconfig_SunSpec_Length,							    // 584 
	FNconfig_Port_Number,								    // 585
	FNconfig_DC_Voltage_SF,								    // 586
	FNconfig_DC_Current_SF,								    // 587
	FNconfig_kWh_SF,									    // 588
	FNconfig_Major_Firmware_Number,						    // 589
	FNconfig_Mid_Firmware_Number,						    // 590
	FNconfig_Minor_Firmware_Number,						    // 591
	FNconfig_Battery_Capacity,							    // 592
	FNconfig_Charged_Volts,								    // 593
	FNconfig_Charged_Time,								    // 594
	FNconfig_Battery_Charged_Amps,						    // 595
	FNconfig_Charge_Factor,								    // 596
	FNconfig_Shunt_A_Enabled,							    // 597
	FNconfig_Shunt_B_Enabled,							    // 598
	FNconfig_Shunt_C_Enabled,							    // 599
	FNconfig_Relay_Control,								    // 600
	FNconfig_Relay_Invert_Logic,						    // 601
	FNconfig_Relay_High_Voltage,						    // 602
	FNconfig_Relay_Low_Voltage,							    // 603
	FNconfig_Relay_SOC_High,							    // 604
	FNconfig_Relay_SOC_Low,								    // 605
	FNconfig_Relay_High_Enable_Delay,					    // 606
	FNconfig_Relay_Low_Enable_Delay,					    // 607
	FNconfig_Set_Data_Log_Day_Offset,					    // 608
	FNconfig_Get_Current_Data_Log_Day_Offset,			    // 609
	FNconfig_Datalog_Minimum_SOC,						    // 610
	FNconfig_Datalog_Input_AH,							    // 611
	FNconfig_Datalog_Input_kWh,							    // 612
	FNconfig_Datalog_Output_AH,							    // 613
	FNconfig_Datalog_Output_kWh,						    // 614
	FNconfig_Datalog_NET_AH,							    // 615
	FNconfig_Datalog_NET_kWh,							    // 616
	FNconfig_Clear_Data_Log_Read,						    // 617
	FNconfig_Clear_Data_Log_Write_Complement,			    // 618
	FNconfig_Serial_Number,								    // 619
	FNconfig_Model_Number,								    // 620
	
	FN_SunSpec_DID,										    // 621
	FN_SunSpec_Length,									    // 622
	FN_Port_Number,										    // 623
	FN_DC_Voltage_SF,									    // 624
	FN_DC_Current_SF,									    // 625
	FN_Time_SF,											    // 626
	FN_kWh_SF,											    // 627	
	FN_kW_SF,												// 628 
	FN_Shunt_A_Current,									    // 629
	FN_Shunt_B_Current,									    // 630
	FN_Shunt_C_Current,									    // 631
	FN_Battery_Voltage,									    // 632
	FN_Battery_Current,									    // 633
	FN_Battery_Temperature,								    // 634
	FN_Status_Flags,									    // 635
	FN_Shunt_A_Accumulated_AH,							    // 636
	FN_Shunt_A_Accumulated_kWh,							    // 637
	FN_Shunt_B_Accumulated_AH,							    // 638
	FN_Shunt_B_Accumulated_kWh,							    // 639
	FN_Shunt_C_Accumulated_AH,							    // 640
	FN_Shunt_C_Accumulated_kWh,							    // 641	
	FN_Input_Current,										// 642
	FN_Output_Current,										// 643
	FN_Input_kW,											// 644
	FN_Output_kW,											// 645
	FN_Net_kW,												// 646	
 	FN_Days_Since_Charge_Parameters_Met,				    // 647
	FN_State_Of_Charge,									    // 648
	FN_Todays_Minimum_SOC,								    // 649
	FN_Todays_Maximum_SOC,								    // 650
	FN_Todays_NET_Input_AH,								    // 651
	FN_Todays_NET_Input_kWh,							    // 652
	FN_Todays_NET_Output_AH,							    // 653
	FN_Todays_NET_Output_kWh,							    // 654	
	FN_Todays_NET_Battery_AH,								// 655
	FN_Todays_NET_Battery_kWh,								// 656	
	FN_Charge_Factor_Corrected_NET_Battery_AH,			    // 657
	FN_Charge_Factor_Corrected_NET_Battery_kWh,			    // 658
	FN_Todays_Minimum_Battery_Voltage,					    // 659
	FN_Todays_Minimum_Battery_Time,						    // 660
	FN_Todays_Maximum_Battery_Voltage,					    // 661
	FN_Todays_Maximum_Battery_Time,						    // 662
	FN_Cycle_Charge_Factor,								    // 663
	FN_Cycle_kWh_Charge_Efficiency,						    // 664
	FN_Total_Days_At_100_Percent,						    // 665
	FN_Lifetime_kAH_Removed,							    // 666
	FN_Shunt_A_Historical_Returned_To_Battery_AH,		    // 667
	FN_Shunt_A_Historical_Returned_To_Battery_kWh,		    // 668
	FN_Shunt_A_Historical_Removed_From_Battery_AH,		    // 669
	FN_Shunt_A_Historical_Removed_From_Battery_kWh,		    // 670
	FN_Shunt_A_Maximum_Charge_Rate,						    // 671
	FN_Shunt_A_Maximum_Charge_Rate_kW,					    // 672
	FN_Shunt_A_Maximum_Discharge_Rate,					    // 673
	FN_Shunt_A_Maximum_Discharge_Rate_kW,				    // 674
	FN_Shunt_B_Historical_Returned_To_Battery_AH,		    // 675
	FN_Shunt_B_Historical_Returned_To_Battery_kWh,		    // 676
	FN_Shunt_B_Historical_Removed_From_Battery_AH,		    // 677
	FN_Shunt_B_Historical_Removed_From_Battery_kWh,		    // 678
	FN_Shunt_B_Maximum_Charge_Rate,						    // 679
	FN_Shunt_B_Maximum_Charge_Rate_kW,					    // 680
	FN_Shunt_B_Maximum_Discharge_Rate,					    // 681
	FN_Shunt_B_Maximum_Discharge_Rate_kW,				    // 682
	FN_Shunt_C_Historical_Returned_To_Battery_AH,		    // 683
	FN_Shunt_C_Historical_Returned_To_Battery_kWh,		    // 684
	FN_Shunt_C_Historical_Removed_From_Battery_AH,		    // 685
	FN_Shunt_C_Historical_Removed_From_Battery_kWh,		    // 686
	FN_Shunt_C_Maximum_Charge_Rate,						    // 687
	FN_Shunt_C_Maximum_Charge_Rate_kW,					    // 688
	FN_Shunt_C_Maximum_Discharge_Rate,					    // 689 
	FN_Shunt_C_Maximum_Discharge_Rate_kW,				    // 690
	FN_Shunt_A_Reset_Maximum_Data,						    // 691
	FN_Shunt_A_Reset_Maximum_Data_Write_Complement,		    // 692
	FN_Shunt_B_Reset_Maximum_Data,						    // 693
	FN_Shunt_B_Reset_Maximum_Data_Write_Complement,		    // 694
	FN_Shunt_C_Reset_Maximum_Data,						    // 695
	FN_Shunt_C_Reset_Maximum_Data_Write_Complement,		    // 696
	
	A_SunSpec_DID,										    // 697 
	A_SunSpec_Length,									    // 698 
	A_Devices,											    // 699
	A_Count,											    // 700 
	A_Update_Number,									    // 701 
	A_Status,											    // 702 
	A_Status_Vendor,									    // 703 
	A_Event,											    // 704 
	A_Event_Vendor,										    // 705 
	A_Control,											    // 706 
	A_Control_Vendor,									    // 707 
	A_Control_Value,									    // 708 
	END_SunSpec_DID,									    // 709 
	END_SunSpec_Length									    // 710 
} SunSpecField;


extern const char* fnames[];

int				outbackInit			(char *, int);
void			outbackClose		(void);
uint16			getBlockType		(int block);
uint64			getField			(int, SunSpecField, char *);
int				setFieldWithString	(int, SunSpecField, char *);
int				setFieldsWithString	(int, SunSpecField, char *);
int				fieldIsInBlock		(SunSpecField, int);
SunSpecField	firstField		(	int block);
SunSpecField	lastField			(int block);

#endif
